import pygame
import math
from player import BasicAttack, HomingAttack, IFall

def isHurtState(state):
	if isinstance(state, BasicAttack) or isinstance(state, HomingAttack) or isinstance(state, IFall):
		return True
	return False

def loadImage(filename):
		image = pygame.image.load(filename)
		image = image.convert()
		image.set_colorkey((255,0,255))
		return image

ramen = []
coffee = []
explosion = []
missile = []
bomb = None
def initialiseSprites():
	global bomb
	for i in range(1,16):
		path = "data//images//ramen//ramen" + str(i) + ".png"
		ramen.append(loadImage(path))
	for i in range(1,5):
		path = "data//images//coffee//coffee" + str(i) + ".png"
		coffee.append(loadImage(path))
	for i in range(1,10):
		path = "data//images//explosion//ex" + str(i) + ".png"
		explosion.append(loadImage(path))
	for i in range(1,3):
		path = "data//images//platform//missile//missile" + str(i) + ".png"
		missile.append(loadImage(path))
	bomb = loadImage("data//images//bomber//bomb//bomb.png")
	

class Item(object):
	def __init__(self,screen,pos):
		self.currentAnim = 0
		self.screen = screen
		self.pos = pos
		self.veloc = 0
		self.acc = 2
		self.frame = 0
		self.rect = pygame.Rect(0,0,10,10)
		self.rect.center = pos
		self.drawrect = self.rect.copy()		
		self.alive = True
	def update(self):
		self.frame += 1
		self.veloc += self.acc
		self.pos = (self.pos[0], self.pos[1] + self.veloc/4.0)

		if self.pos[1] > 292:
			self.pos = (self.pos[0], 292)	
			self.acc = 0
		self.rect.centery = int(math.sin(self.frame*0.3)*4)+self.pos[1]		
	
	def draw(self,background):
		self.drawrect.center = (self.rect.center[0] - background.xpos, self.rect.center[1])
		pygame.draw.circle(self.screen, (255,0,0), self.drawrect.center, 4)
	def collidePlayer(self,player):
		if self.rect.colliderect(player.rect):
			self.alive = False			
			#player.intChnl.play(player.sounds[5])	
			
class Coffee(Item):
	def draw(self,background):
		im = coffee[self.currentAnim]
		self.drawrect = im.get_rect()
		self.drawrect.center = (self.rect.center[0] - background.xpos, self.rect.center[1]-5)
		self.screen.blit(im,self.drawrect)		
		if self.frame % 4 == 0:
			self.currentAnim = (self.currentAnim+ 1) % len(coffee)
	def collidePlayer(self,player):
		if self.rect.colliderect(player.rect):
			self.alive = False
			player.addPower()

class Ramen(Item):
	def draw(self,background):
		im = ramen[self.currentAnim]
		self.drawrect = im.get_rect()
		self.drawrect.center = (self.rect.center[0] - background.xpos, self.rect.center[1]-5)
		self.screen.blit(im,self.drawrect)
		if self.frame % 2 == 0:
			self.currentAnim = (self.currentAnim+ 1) % len(ramen)
			
	def collidePlayer(self,player):
		if self.rect.colliderect(player.rect):
			self.alive = False
			player.heal()

class Laser(Item):
	def __init__(self,screen,pos,veloc):
		self.pos = pos
		self.screen = screen
		self.veloc = veloc
		self.drawrect = pygame.Rect(0,0,10,10)
		self.alive = True
	def update(self):		
		self.pos = (self.pos[0]+self.veloc[0],self.pos[1]+self.veloc[1])
		
	def draw(self,background):		
		self.drawrect.center = (self.pos[0] - background.xpos, self.pos[1])
		dpos = (self.pos[0]-background.xpos,self.pos[1])
		npos = (self.pos[0]-self.veloc[0]-background.xpos,self.pos[1]-self.veloc[1])
		pygame.draw.line(self.screen, (255,45,0), dpos, npos,2)
	def collidePlayer(self,player):
		if player.rect.collidepoint(self.pos):			
			if player.state.type != "ATTACK" or isHurtState(player.state):
				player.hurt()
			self.alive = False

class Bomb(Item):
	def __init__(self,screen,pos):
		Item.__init__(self,screen,pos)
		self.image = bomb		
		self.drawrect = self.image.get_rect()
		self.rect = self.image.get_rect()
		self.rect.center = self.pos
		self.veloc = 6
		self.mode = "FALL"

	def draw(self,background):
		self.drawrect.center = (self.rect.center[0] - background.xpos, self.rect.center[1])
		if self.mode == "EXPLODE" or self.mode == "DIE":
			self.image = explosion[int(self.frame/2)]
		self.screen.blit(self.image,self.drawrect)
	
	def update(self):	

		self.frame += 1
		self.veloc += 2
		
		if self.mode == "FALL":
			self.rect.bottom += self.veloc/4.0
			if self.rect.bottom > 300:
				self.rect.bottom = 300
				self.mode = "EXPLODE"
				self.frame = 0
				self.rect = explosion[0].get_rect()
				self.drawrect = explosion[0].get_rect()
				self.rect.center = self.pos
				self.rect.centery = 296
			
		if self.mode == "EXPLODE" or self.mode == "DIE":			
			if self.frame > 16:
				self.alive = False
		self.pos = self.rect.center
	def collidePlayer(self,player):
		if self.rect.colliderect(player.rect):
			if self.mode == "EXPLODE":
				self.mode = "DIE"
				if player.state.type != "ATTACK" or isHurtState(player.state):
					player.hurt()
			elif self.mode == "FALL":
				self.mode = "EXPLODE"
				self.frame = 0
				self.rect = explosion[0].get_rect()
				self.drawrect = explosion[0].get_rect()
				self.rect.center = self.pos

class Missile(Item):
	def __init__(self,screen,pos):
		Item.__init__(self,screen,pos)
		
		self.drawrect = missile[0].get_rect()
		self.rect = missile[0].get_rect()
		self.rect.center = self.pos
		self.veloc = 8
		self.yveloc = 0
		self.mdir = 0
		self.mode = "FIRE"
		self.currentAnim = 0

	def draw(self,background):
		self.drawrect.center = (self.rect.center[0] - background.xpos, self.rect.center[1])
		if self.mode == "EXPLODE" or self.mode == "DIE":
			self.image = explosion[int(self.frame/2)]
		else:
			self.image = missile[self.currentAnim]
		self.screen.blit(self.image,self.drawrect)
	
	def update(self):	
		if self.frame % 3 == 0:
			self.currentAnim = (self.currentAnim+ 1) % 2
		self.frame += 1
		self.veloc += 2
		yfric = -self.yveloc*0.4
		self.yveloc += self.mdir + yfric 
		
		if self.mode == "FIRE":
			self.rect.centerx -= self.veloc/4.0
			self.rect.centery += self.yveloc/4.0
			
		if self.mode == "EXPLODE" or self.mode == "DIE":			
			if self.frame > 16:
				self.alive = False
		self.pos = self.rect.center
	def collidePlayer(self,player):
		self.mdir = math.copysign(2, (player.pos[1] - self.pos[1]))
		if self.rect.colliderect(player.rect):			
			if self.mode == "EXPLODE":
				self.mode = "DIE"
				if player.state.type != "ATTACK" or isHurtState(player.state):
					player.hurt()
			elif self.mode == "FIRE":
				self.mode = "EXPLODE"
				self.frame = 0
				self.rect = explosion[0].get_rect()
				self.drawrect = explosion[0].get_rect()
				self.rect.center = self.pos