import pygame 
import random
def loadImage(filename):
		image = pygame.image.load(filename)
		image = image.convert()
		image.set_colorkey((255,0,255))
		return image

class Player():
	def __init__(self,screen):
		self.screen = screen
		self.rect = pygame.Rect(0,0,15,20)
		self.rect.bottomright = (64,300)
		self.pos = self.rect.center
		self.transforming = False
		self.tf = 0 # Transform Frame
		self.state = Move()
		self.leftmost = 5
		self.action = True
		self.af = 0
		self.closest = None

		self.stopcharge = False
		
		# attributes
		self.big = False
		self.health = 27 #27 at max
		self.power = 0 # When it reaches 270 you can use the power
		self.score = 0 # keep it here for best access
		
		self.multiplier = 1
		self.mtime = 0
		
		self.invincible = 0
		
		# Important - attack num determines the current number of the attack so that enemies dont get damaged by the same attack twice
		self.attackNum = 0
		
		self.moveChnl = pygame.mixer.Channel(1)
		self.attkChnl = pygame.mixer.Channel(2)
		self.intChnl = pygame.mixer.Channel(3)
		# Scale volumes
		self.moveChnl.set_volume(0.3)	
		self.attkChnl.set_volume(0.3)
		self.intChnl.set_volume(0.3)			
		self.sounds = []
		self.sounds.append(pygame.mixer.Sound("data//sound//attack.wav"))
		self.sounds.append(pygame.mixer.Sound("data//sound//blast.wav"))
		self.sounds.append(pygame.mixer.Sound("data//sound//charge.wav"))
		self.sounds.append(pygame.mixer.Sound("data//sound//fly.wav"))
		self.sounds.append(pygame.mixer.Sound("data//sound//gethealth.wav"))
		self.sounds.append(pygame.mixer.Sound("data//sound//getpower.wav"))
		self.sounds.append(pygame.mixer.Sound("data//sound//hurt.wav"))		
		self.sounds.append(pygame.mixer.Sound("data//sound//grow.wav"))
		self.sounds.append(pygame.mixer.Sound("data//sound//shrink.wav"))
		self.sounds.append(pygame.mixer.Sound("data//sound//powerup.wav"))
		self.sounds.append(pygame.mixer.Sound("data//sound//powerupdone.wav"))
		self.setupAnimations()
	
	def multiply(self):
		self.multiplier += 1
		self.mtime = 0
	
	def setupAnimations(self):
		# Load all the character animation images
		# Again very messy but i cant think of a better way in the current time limits
		self.images = []
		self.images.append(loadImage('data//images//idle//idle1.png')) #0
		self.images.append(loadImage('data//images//idle//idle2.png'))
		self.images.append(loadImage('data//images//run//run1.png'))
		self.images.append(loadImage('data//images//run//run2.png'))
		self.images.append(loadImage('data//images//run//run3.png'))
		self.images.append(loadImage('data//images//jump//jump1.png')) #5
		self.images.append(loadImage('data//images//jump//jump2.png'))
		self.images.append(loadImage('data//images//fall//fall.png'))
		self.images.append(loadImage('data//images//crouch//crouch.png'))
		self.images.append(loadImage('data//images//fallhurt//fallhurt1.png'))
		self.images.append(loadImage('data//images//fallhurt//fallhurtred.png')) #10
		self.images.append(loadImage('data//images//idlecharge//idlecharge.png'))
		self.images.append(loadImage('data//images//idlecharge//idlecharge2.png'))
		self.images.append(loadImage('data//images//idlesupercharge//idlesupercharge1.png'))
		self.images.append(loadImage('data//images//idlesupercharge//idlesupercharge2.png'))
		self.images.append(loadImage('data//images//hovercharge//hovercharge1.png')) # 15
		self.images.append(loadImage('data//images//hovercharge//hovercharge2.png'))
		self.images.append(loadImage('data//images//hoversuper//hoversuper1.png'))
		self.images.append(loadImage('data//images//hoversuper//hoversuper2.png'))
		self.images.append(loadImage('data//images//crouchcharge//crouchcharge1.png'))
		self.images.append(loadImage('data//images//crouchcharge//crouchcharge2.png')) #20
		self.images.append(loadImage('data//images//crouchsuper//crouchsuper1.png'))
		self.images.append(loadImage('data//images//crouchsuper//crouchsuper2.png'))
		self.images.append(loadImage('data//images//runpunch//runpunch.png'))
		self.images.append(loadImage('data//images//runpunch//runpunch2.png'))
		self.images.append(loadImage('data//images//airdash//airdash1.png')) #25
		self.images.append(loadImage('data//images//airdash//airdash2.png'))
		self.images.append(loadImage('data//images//updash//updash1.png'))
		self.images.append(loadImage('data//images//updash//updash2.png'))
		self.images.append(loadImage('data//images//beam//beam1.png'))
		self.images.append(loadImage('data//images//beam//beam2.png')) #30
		self.images.append(loadImage('data//images//idlesuperchargebeam//idlesuperchargebeam1.png'))
		self.images.append(loadImage('data//images//idlesuperchargebeam//idlesuperchargebeam2.png'))
		self.images.append(loadImage('data//images//grounddash//grounddash1.png'))
		self.images.append(loadImage('data//images//grounddash//grounddash2.png'))
		self.images.append(loadImage('data//images//grounddashsuper//grounddashsuper1.png')) #35
		self.images.append(loadImage('data//images//grounddashsuper//grounddashsuper2.png'))
		
		self.imRect = self.images[0].get_rect()
		self.bigImages = []
		w = self.imRect.width*9
		h = self.imRect.height*9
		for im in self.images:			
			self.bigImages.append(pygame.transform.scale(im,(w,h)))
		self.bigImRect = self.bigImages[0].get_rect()
	def addPower(self):
		if not self.big:
			self.power += 30	
		if self.power > 270:
			self.power = 270
		self.intChnl.play(self.sounds[5])		
	def heal(self):
		self.health = 27
		self.intChnl.play(self.sounds[4])
	def hurt(self):
		if self.invincible <= 0:
			self.multiplier = 1			
			self.action = False
			self.stopcharge = False
			self.af = 0
			self.intChnl.stop()
			self.intChnl.play(self.sounds[6])
			if self.big:
				self.health -= 1
			else:
				self.health -= 3
				self.state = HurtFall(self.state)
				self.rect.centery = self.rect.centery - 2
				self.rect.centerx = self.rect.centerx - 6
			self.invincible = 18
	def update(self,level):
		##########################
		# TRANSFORM CODE
		# MESSILY PLACED HER FOR REASONS OF "THIS COMPETITION IS UNDER A WEEK AND I GOT LAZY!"
		if self.transforming:
			self.state.move(self.rect)
			self.tf += 1
			if self.tf >= 90:
				self.transforming = False
				self.tf = 0				
				print "CHANGE"
				if self.big:				
					self.big = False
					r = pygame.Rect(0,0,15,20)
					self.pos = (self.pos[0], self.pos[1] +(self.rect.height-r.height)/2)
					r.center = self.rect.center
					self.rect = r
				else:
					self.big = True
					r = pygame.Rect(0,0,135,180)
					self.pos = (self.pos[0], self.pos[1] +(self.rect.height-r.height)/2)
					self.rect = r
				self.rect.center = self.pos
				self.state = Fall(0,self.pos,self.state)
				return
			if self.big:				
				r = pygame.Rect(0,0,int((90-self.tf)*4/3.0 + 15),int((90-self.tf)*16/9.0 + 20))				
				self.pos = (self.pos[0], self.pos[1] +(self.rect.height-r.height)/2)
				self.rect = r
			else:
				r = pygame.Rect(0,0,int(self.tf*4/3.0 + 15),int(self.tf*16/9.0 + 20))				
				self.pos = (self.pos[0], self.pos[1] +(self.rect.height-r.height)/2)
				self.rect = r
			self.rect.center = self.pos
			return
		#########################
		
		if self.big:
			self.power -= 1
			if self.power <= 0:				
				self.power = 0
				if not (isinstance(self.state,ChargeBeam) or isinstance(self.state,Explosion)):					
					self.state = Fall(0,self.pos,self.state)
					self.af = 0
					self.action = False
					self.intChnl.stop()
					self.moveChnl.play(self.sounds[8])
					self.transforming = True
		if self.invincible > 0:			
			self.invincible -= 1
		# Reset attack number counter
		if self.attackNum > 200:
			self.attackNum = 0
		if self.action:
			self.updateAction()
		self.state = self.state.update()		
		pos = self.state.move(self.rect)
		#print pos
		if pos[0]-8 < self.leftmost:
			self.pos = (self.leftmost + 8 , pos[1])
		else:
			self.pos = pos
		self.rect.center = self.pos
		self.collideEnemies(level.enemies)
		self.collideItems(level.items)
		
	def collideItems(self,itemlist):
		for item in itemlist:
			item.collidePlayer(self)			
	def collideEnemies(self,enemylist):
		self.closest = None
		for e in enemylist:
			e.collidePlayer(self)		
			if e.pos[0] > self.pos[0] and e.alive and e.pos[1] > 0:
				if self.closest == None:
					self.closest = e
				else:
					if e.pos[0]+e.pos[1] < self.closest.pos[0]+self.closest.pos[1]:						
						self.closest = e				
				
			
	def updateAction(self):
		if self.state.type == "MOVE" or self.state.type == "JUMP" or self.state.type == "CROUCH" or self.state.type == "CHARGE":
			self.af += 1		
		if self.stopcharge and self.af >= 25:			
			self.af = 0
			self.action = False
			self.intChnl.stop()
			self.state = self.state.getAttack()
			self.stopcharge = False
		if self.af >= 10:			
			if self.state.type == "MOVE" or isinstance(self.state,BasicAttack):				
				self.state = ChargeGround(self.state,self)
			elif self.state.type == "JUMP" or isinstance(self.state,HomingAttack):
				self.state = ChargeFall(self.state,self)
			elif self.state.type == "CROUCH":
				self.state = ChargeCrouch(self.state,self)
			else:
				return
			self.intChnl.play(self.sounds[9])
	def draw(self,background):		
		self.leftmost = background.xpos+5
		drawrect = self.rect.copy()
		if not self.big:
			drawrect.center = (self.pos[0] - background.xpos, self.pos[1]-5)			
			imageNum, imagePos = self.state.draw(drawrect,self.screen)		
			self.imRect.center = imagePos
		else:
			drawrect.center = (self.pos[0] - background.xpos, self.pos[1]-45)
			imageNum, imagePos = self.state.draw(drawrect,self.screen)	
			self.bigImRect.center = imagePos
		if self.transforming:
			s = ((drawrect.width*26)/15,(drawrect.height*32)/20)
			im = pygame.transform.scale(self.images[imageNum],s)			
			r = im.get_rect()
			r.center = drawrect.center
			self.screen.blit(im,r)
			return
		if not self.big:
			self.screen.blit(self.images[imageNum],self.imRect)
		else:
			self.screen.blit(self.bigImages[imageNum],self.bigImRect)
	def move(self,direction):
		if self.state.type == "MOVE":
			self.state.XMove = direction
		else:
			self.state.AMove = direction
	def stop(self,direction):
		if self.state.type == "MOVE":
			if self.state.XMove == direction:
				self.state.XMove = 0			
		else:
			if self.state.AMove == direction:
				self.state.AMove = 0
	def jump(self):
		if self.state.type == "MOVE":
			self.moveChnl.play(self.sounds[3])
			if self.big:
				self.state = Fall(-40,self.pos,self.state,5)
			else:
				self.state = Fall(-30,self.pos,self.state,5)
		
		elif self.state.type == "JUMP":
			if self.power == 270:				
				# Perform transform here
				self.moveChnl.play(self.sounds[7])
				self.transforming = True
				#pass

			
	def crouch(self):
		if self.state.type == "MOVE":
			self.state = Crouch(self.state)
	def uncrouch(self):		
		if self.state.type == "CROUCH":
			m = Move()
			m.XMove = self.state.AMove
			self.state = m
	def saction(self):
		#if self.state.type == "CROUCH" or self.state.type == "MOVE" or self.state.type == "JUMP":
		self.action = True
		if self.state.type == "MOVE":
			print "POWER PUNCH"
			self.attkChnl.play(self.sounds[0])
			self.state = BasicAttack(self,self.attackNum)
			self.attackNum += 1	

	def deaction(self):

		if self.state.type == "CHARGE":
			self.stopcharge = True
		
		else:
			self.af = 0
			self.action = False	
			print "ATTACK!"
			if self.state.type == "MOVE":
				print "POWER PUNCH"
				self.attkChnl.play(self.sounds[0])
				self.state = BasicAttack(self,self.attackNum)
				self.attackNum += 1		
			if (self.state.type == "JUMP" or isinstance(self.state,IFall)) and self.closest != None:			
				print "HOMING PUNCH"
				self.attkChnl.play(self.sounds[3])
				self.state = HomingAttack(self,self.attackNum,self.closest)
				self.attackNum += 1	
			if self.state.type == "CROUCH":			
				print "BLAST JUMP"
				self.moveChnl.play(self.sounds[3])
				self.attkChnl.play(self.sounds[0])
				
				self.state = BlastJump(self,self.attackNum)
				
				self.attackNum += 1
			
class AttackState(object):
	def __init__(self,player,num,power):
		self.AMove = player.state.AMove
		self.type = "ATTACK"
		self.power = power
		
		self.colour = (127,0,0)
		self.atknum = num
		self.frame = 0
		self.player = player
		self.knockback = False
		self.burn = False
		self.currentAnim = 23
	def getCollisionState(self,enemy):
		return self
	def getCollisionDamage(self):
		if self.player.big:
			return self.power * 3
		else:
			return self.power
	def move(self,rect):
		return (rect.center)	
	def update(self):		
		self.frame += 1
		return self		
	def draw(self,drawrect,screen):
		return self.currentAnim, drawrect.center

		
		
class BasicAttack(AttackState):
	def __init__(self,player,num):
		AttackState.__init__(self,player,num,5)
		self.XMove = player.state.AMove
		self.XMove = 1
		self.frame = 0
		self.knockback = True
		self.currentAnim = 23
	def getCollisionState(self,enemy):
		if enemy.health > 0:
			self.player.rect.centerx = self.player.rect.centerx-15
			m = CMove()
			m.XMove = self.AMove			
			return m
		return self	
	def update(self):
		self.frame += 1
		if self.frame % 2 ==0:
			if self.currentAnim == 23:
				self.currentAnim = 24
			else:
				self.currentAnim = 23
		if self.frame > 6:
			m = CMove(5)
			m.XMove = self.AMove			
			return m
		return self
	def move(self,rect):
		return (rect.centerx+self.XMove*12, rect.centery)

class HomingAttack(AttackState):
	def __init__(self,player,num,target):
		AttackState.__init__(self,player,num,5)
		self.target = target
		self.frame = 0
		self.knockback = True
		self.currentAnim = 25
	def getCollisionState(self,enemy):	
		self.player.state = HurtFallTwo(self)
		self.player.invincible = 6	
		self.frame = 30
		return self
	def update(self):
		if self.frame % 2 ==0:
			if self.currentAnim == 25:
				self.currentAnim = 26
			else:
				self.currentAnim = 25
		if self.frame > 30 or self.target.health <= 0:
			self.player.rect.centerx = self.player.rect.centerx - 16		
			return HurtFallTwo(self)		
		self.frame += 1
		return self
	def move(self,rect):
		cx = rect.centerx
		cy = rect.centery
		dx = -cx + self.target.pos[0]
		dy = -cy + self.target.pos[1]
		d = ((dx**2 + dy**2)**0.5)+0.01
		nx = cx + int(18*(dx/d))
		ny = cy + int(18*(dy/d))		
		if ny > 300:
			ny = 300
		return (nx, ny)

class BlastJump(AttackState):
	def __init__(self,player,num):
		AttackState.__init__(self,player,num,15)
		self.frame = 0
		c = self.player.rect.center
		if self.player.big:
			self.player.rect.width = 256
			self.player.rect.height = 160
		else:		
			self.player.rect.width = 64
			self.player.rect.height = 32
		self.player.rect.center = c
		self.burn = True
		self.currentAnim = 27
		self.explosions = []
		self.explosions.append(loadImage('data//images//explosionground//exg1.png'))
		self.explosions.append(loadImage('data//images//explosionground//exg2.png'))
		self.explosions.append(loadImage('data//images//explosionground//exg3.png'))
		self.explosions.append(loadImage('data//images//explosionground//exg4.png'))
		self.explosions.append(loadImage('data//images//explosionground//exg5.png'))
		self.explosions.append(loadImage('data//images//explosionground//exg6.png'))
		self.exRect = self.explosions[0].get_rect()
		if player.big:
			self.exRect.width = self.exRect.width * 4
			self.exRect.height = self.exRect.height * 4
			for e in range(len(self.explosions)):
				self.explosions[e] = pygame.transform.scale(self.explosions[e],self.exRect.size)
	def update(self):
		self.frame += 1
		if self.frame % 2 ==0:
			if self.currentAnim == 27:
				self.currentAnim = 28
			else:
				self.currentAnim = 27
		if self.frame > 5:
			c = self.player.rect.center
			if self.player.big:
				self.player.rect.width = 135
				self.player.rect.height = 180
			else:		
				self.player.rect.width = 15
				self.player.rect.height = 20
			self.player.rect.center = c
			self.player.attackNum += 1	
			f = IFall(-30,self.player,self,self.player.attackNum)
			f.AMove = self.AMove		
			return f		
		return self
	def move(self,rect):
		return (rect.centerx, rect.centery-5)
	def draw(self,drawrect,screen):
		self.exRect.bottom = 300
		self.exRect.centerx = drawrect.centerx
		screen.blit(self.explosions[self.frame],self.exRect)
		return self.currentAnim, drawrect.center
		
class ChargeBlastJump(AttackState):
	def __init__(self,player,num,charge):
		dmg = 8
		if charge > 40:
			dmg = 15	
		AttackState.__init__(self,player,num,dmg)
		if charge > 40:
			self.currentAnim = 35
		else:
			self.currentAnim = 33
		self.frame = 0
		self.limit = (charge*2)/3
		self.knockback = True
	def getCollisionState(self,enemy):
		if enemy.health > 0:
			self.player.rect.centerx = self.player.rect.centerx-15
			self.player.moveChnl.play(self.player.sounds[3])
			self.player.attkChnl.play(self.player.sounds[0])
			b = BlastJump(self.player,self.player.attackNum)
			self.player.attackNum += 1			
			return b
		return self
	def update(self):
		self.frame += 1
		if self.frame % 3 == 0:	
			if self.currentAnim == 33:
				self.currentAnim = 34
			elif self.currentAnim == 34:
				self.currentAnim = 33
			elif self.currentAnim == 35:
				self.currentAnim = 36
			else:
				self.currentAnim = 35
		if self.frame > self.limit:			
			self.player.moveChnl.play(self.player.sounds[3])
			self.player.attkChnl.play(self.player.sounds[0])
			b = BlastJump(self.player,self.player.attackNum)
			self.player.attackNum += 1			
			return b
		return self
	def move(self,rect):
		return (rect.centerx+15, rect.centery)

class ChargeBeam(AttackState):
	def __init__(self,player,num,charge):
		AttackState.__init__(self,player,num,int((charge**1.5)/8.0))
		if charge > 40:
			self.currentAnim = 31
		else:
			self.currentAnim = 29
		c = self.player.rect.left
		self.player.rect.width = 640	
		self.player.pos = self.player.rect.center
		self.player.rect.left = c		
		self.frame = 0
		self.limit = (charge**1.5)/5
		self.burn = True
		self.beam = loadImage('data//images//beamfx//beam.png')
		self.beamRect = self.beam.get_rect()
		if self.player.big:
			self.beam = loadImage('data//images//beamfx//beambig.png')
		self.beamRect.centery = self.player.rect.centery
	def update(self):
		self.frame += 1		
		if self.frame % 3 == 0:	
			if self.currentAnim == 29:
				self.currentAnim = 30
			elif self.currentAnim == 30:
				self.currentAnim = 29
			elif self.currentAnim == 31:
				self.currentAnim = 32
			else:
				self.currentAnim = 31
		if self.frame > self.limit:	
			c = self.player.rect.centery
			if self.player.big:
				self.player.rect.width = 135
			else:		
				self.player.rect.width = 15
			self.player.rect.centery = c		
			m = CMove(20)
			m.XMove = self.AMove			
			return m
		return self
	def move(self,rect):
		if not self.player.big:
			return (rect.centerx+int(random.random()*4)-2, rect.centery)
		else:
			return (rect.centerx+int(random.random()*36)-18, rect.centery)
	def draw(self,drawrect,screen):		
		self.beamRect.centery = drawrect.centery
		if not self.player.big:			
			self.beamRect.centery += int(random.random()*3)-1
			self.beamRect.centerx = drawrect.centerx + 15
			screen.blit(self.beam,self.beamRect)
			return self.currentAnim, (drawrect.centerx-312,drawrect.centery)
		else:						
			self.beamRect.centery += int(random.random()*5)-2-120
			self.beamRect.centerx = drawrect.centerx + 135
			screen.blit(self.beam,self.beamRect)
			return self.currentAnim, (drawrect.centerx-252,drawrect.centery)
class Explosion(AttackState):
	def __init__(self,player,num,charge):
		p = 20
		if charge > 40:
			p = 30		
		AttackState.__init__(self,player,num,p)
		if charge > 40:
			self.currentAnim = 17
		else:
			self.currentAnim = 15
		self.frame = 0
		self.limit = (charge+5)/2
		self.burn = True
		self.explosions = []
		self.explosions.append(loadImage('data//images//explosion//ex1.png'))
		self.explosions.append(loadImage('data//images//explosion//ex2.png'))
		self.explosions.append(loadImage('data//images//explosion//ex3.png'))
		self.explosions.append(loadImage('data//images//explosion//ex4.png'))
		self.explosions.append(loadImage('data//images//explosion//ex5.png'))
		self.explosions.append(loadImage('data//images//explosion//ex6.png'))		
		self.explosions.append(loadImage('data//images//explosion//ex7.png'))
		self.explosions.append(loadImage('data//images//explosion//ex8.png'))
		self.explosions.append(loadImage('data//images//explosion//ex9.png'))


	def update(self):
		self.frame += 1
		if self.frame % 3 == 0:	
			if self.currentAnim == 15:
				self.currentAnim = 16
			elif self.currentAnim == 16:
				self.currentAnim = 15
			elif self.currentAnim == 17:
				self.currentAnim = 18
			else:
				self.currentAnim = 17
		c = self.player.rect.center
		if self.player.big:
			m = 92
		else:
			m = 24
		self.player.rect.width = 16 + self.frame * m
		self.player.rect.height = 16 + self.frame * m
		self.player.rect.center = (c[0]-1,c[1])			
		if self.frame > self.limit:	
			c = self.player.rect.center
			if self.player.big:
				self.player.rect.width = 135
				self.player.rect.height = 180
			else:		
				self.player.rect.width = 15
				self.player.rect.height = 20
			self.player.rect.center = c		
			f = Fall(0,self.player.pos,self)
			f.AMove = self.AMove
			return f	
		return self
	def move(self,rect):
		return (rect.centerx+int(random.random()*3)-1, rect.centery+int(random.random()*3)-1)
	def draw(self,drawrect,screen):
		self.exRect = drawrect.copy()
		frame = int((self.frame*9)/(self.limit+1))
		e = pygame.transform.scale(self.explosions[frame],self.exRect.size)
		screen.blit(e,self.exRect)
		return self.currentAnim, drawrect.center		
class IFall(AttackState):	
	def __init__(self,veloc,player,ps,num):
		AttackState.__init__(self,player,num,5)
		self.AMove = ps.AMove
		self.XMove = ps.AMove
		self.veloc = veloc
		self.stop = False
		self.type = "ATTACK"
		self.currentAnim = 27
	def move(self,rect):		
		r = rect.copy()
		mx = (self.XMove)*5 + self.AMove*2
		r.bottom = r.bottom + self.veloc/4.0
		if r.bottom > 300:		
			self.stop = True
			r.bottom = 300
		return (r.center[0] +mx,r.center[1])
		
	def update(self):
		if self.frame % 2 ==0:
			if self.currentAnim == 27:
				self.currentAnim = 28
			else:
				self.currentAnim = 27
		self.frame += 1
		if self.frame > 15:
			f = Fall(0,self.player.pos,self)
			f.AMove = self.AMove
			return f		
		self.veloc += 2
		if self.stop:					
			m = Move()
			m.XMove = self.AMove
			return m
		else:
			return self

class ChargeFall(object):
	def __init__(self,ps,player):
		self.AMove = ps.AMove
		self.type = "CHARGE"
		self.frame = 0
		self.player = player
		self.currentAnim = 15
	def move(self,rect):
		return (rect.center)		
	def update(self):
		self.frame += 1
		if self.frame == 45:
			self.currentAnim = 17
			self.player.moveChnl.play(self.player.sounds[10])			
		if self.frame % 3 == 0:	
			if self.currentAnim == 15:
				self.currentAnim = 16
			elif self.currentAnim == 16:
				self.currentAnim = 15
			elif self.currentAnim == 17:
				self.currentAnim = 18
			else:
				self.currentAnim = 17
		return self
	def draw(self,drawrect,screen):
		p = self.frame/7
		if p > 9:
			p = 20
		return self.currentAnim, drawrect.center
		
	def getAttack(self):
		p = (self.frame)/3 + 15
		if p > 29:
			p = 54
		print "CHARGE BLAST JUMP"
		self.player.moveChnl.play(self.player.sounds[1])							
		s = Explosion(self.player,self.player.attackNum,p)	
		self.player.attackNum += 1		
		return s		

class ChargeGround(object):
	def __init__(self,ps,player):
		self.AMove = ps.AMove
		self.type = "CHARGE"
		self.frame = 0 
		self.currentAnim = 11	
		self.player = player
	def move(self,rect):
		m = Move()
		m.XMove = m.AMove
		return (rect.center)		
	def update(self):
		self.frame += 1
		if self.frame == 45:
			self.currentAnim = 13
			self.player.moveChnl.play(self.player.sounds[10])			
		if self.frame % 3 == 0:	
			if self.currentAnim == 11:
				self.currentAnim = 12
			elif self.currentAnim == 12:
				self.currentAnim = 11
			elif self.currentAnim == 13:
				self.currentAnim = 14
			else:
				self.currentAnim = 13
		return self
	def draw(self,drawrect,screen):
		return self.currentAnim, drawrect.center
		
	def getAttack(self):
		p = (self.frame)/3 + 15
		if p > 29:
			p = 54
		print "CHARGE BLAST JUMP"
		self.player.moveChnl.play(self.player.sounds[1])							
		s = ChargeBeam(self.player,self.player.attackNum,p)	
		self.player.attackNum += 1		
		return s		

class ChargeCrouch(object):
	def __init__(self,ps,player):
		self.AMove = ps.AMove
		self.type = "CHARGE"
		self.frame = 0
		self.player = player
		self.currentAnim = 19
	def move(self,rect):
		return (rect.center)		
	def update(self):
		self.frame += 1
		if self.frame == 45:
			self.currentAnim = 21
			self.player.moveChnl.play(self.player.sounds[10])			
		if self.frame % 3 == 0:	
			if self.currentAnim == 19:
				self.currentAnim = 20
			elif self.currentAnim == 20:
				self.currentAnim = 19
			elif self.currentAnim == 21:
				self.currentAnim = 22
			else:
				self.currentAnim = 21
		return self
	def draw(self,drawrect,screen):
		p = self.frame/7.0
		if p > 6:
			p = 20
		return self.currentAnim, drawrect.center
	
	def getAttack(self):
		p = (self.frame)/3 + 15
		if p > 34:
			p = 54
		print "CHARGE BLAST JUMP"
		self.player.moveChnl.play(self.player.sounds[2])							
		s = ChargeBlastJump(self.player,self.player.attackNum,p)	
		self.player.attackNum += 1		
		return s

class HurtFall(object):
	def __init__(self,ps):
		self.AMove = ps.AMove
		self.XMove = -0.5
		if ps.type == "JUMP":
			self.veloc = (abs(ps.veloc)+ps.veloc)/2
		self.veloc = 0
		self.stop = False
		self.type = "HURT"
		self.f = 0
		self.currentAnim = 9
	def move(self,rect):
		r = rect.copy()
		mx = (self.XMove)*5 + self.AMove
		r.bottom = r.bottom + self.veloc/4.0
		if r.bottom > 300:		
			self.stop = True
			r.bottom = 300
		return (r.center[0] +mx,r.center[1])
		
	def update(self):
		self.f += 1
		if self.f % 2 == 0:
			if self.currentAnim == 9:
				self.currentAnim = 10
			else:
				self.currentAnim = 9
		self.veloc += 2
		if self.stop and self.f > 10:					
			m = CMove(2)
			m.AMove = self.AMove
			return m
		else:
			return self
	def draw(self,drawrect,screen):
		return self.currentAnim, drawrect.center

class HurtFallTwo(object):
	def __init__(self,ps):
		self.AMove = ps.AMove
		self.XMove = -0.5
		if ps.type == "JUMP":
			self.veloc = (abs(ps.veloc)+ps.veloc)/2
		self.veloc = 0
		self.stop = False
		self.type = "HURT"
		self.f = 0
		self.currentAnim = 7
	def move(self,rect):
		r = rect.copy()
		mx = (self.XMove)*5 + self.AMove
		r.bottom = r.bottom + self.veloc/4.0
		if r.bottom > 300:		
			self.stop = True
			r.bottom = 300
		return (r.center[0] +mx,r.center[1])
		
	def update(self):
		self.f += 1
		self.veloc += 2
		if self.stop and self.f > 7:					
			m = CMove(2)
			m.AMove = self.AMove
			return m
		else:
			return self
	def draw(self,drawrect,screen):
		return self.currentAnim, drawrect.center
		
class Fall(object):
	def __init__(self,veloc,pos,ps,curAnim = 6):
		self.AMove = ps.AMove
		self.XMove = ps.AMove
		self.veloc = veloc
		self.stop = False
		self.type = "JUMP"
		self.frame = 0
		self.currentAnim = curAnim
	def move(self,rect):
		r = rect.copy()
		mx = (self.XMove)*5 + self.AMove*2
		r.bottom = r.bottom + self.veloc/4.0
		if r.bottom > 300:		
			self.stop = True
			r.bottom = 300
		return (r.center[0] +mx,r.center[1])
		
	def update(self):
		self.frame += 1
		if self.frame > 5:
			self.currentAnim = 6
		if self.veloc > 0:
			self.currentAnim = 7
		self.veloc += 2
		if self.stop:					
			m = Move()
			m.XMove = self.AMove
			return m
		else:
			return self
	def draw(self,drawrect,screen):
		return self.currentAnim, drawrect.center

		
class Move(object):
	def __init__(self):
		self.XMove = 0
		self.AMove = 0
		self.oldXMove = 0		
		self.type = "MOVE"
		self.frame = 0
		self.currentAnim = 0
	def move(self,rect):
		r = rect.copy()
		mx = (self.XMove+self.oldXMove)*3
		self.oldXMove = self.XMove
		return (r.center[0] +mx,r.center[1])
	def update(self):
		self.frame += 1
		if self.frame % 5 == 0:
			if self.AMove == 0:		
				if self.currentAnim == 0:
					self.currentAnim = 1
				else:
					self.currentAnim = 0		
		if self.frame % 3 == 0:		
			if self.AMove == 1:
				if self.currentAnim == 2:
					self.currentAnim = 3
				elif self.currentAnim == 3:
					self.currentAnim = 4
				else:
					self.currentAnim = 2
			if self.AMove == -1:
				if self.currentAnim == 2:
					self.currentAnim = 4
				elif self.currentAnim == 4:
					self.currentAnim = 3
				else:
					self.currentAnim = 2
					
		self.AMove = self.XMove
		return self
	def draw(self,drawrect,screen):		
		return self.currentAnim, drawrect.center

class CMove(object):
	def __init__(self,limit = 6):
		self.XMove = 0
		self.AMove = 0
		self.oldXMove = 0		
		self.type = "COOLMOVE"
		self.frame = 0
		self.limit = limit
		self.currentAnim = 0
	def move(self,rect):
		r = rect.copy() 
		mx = (self.AMove+self.oldXMove)*3
		self.oldXMove = self.AMove
		return (r.center[0] +mx,r.center[1])
	def update(self):
		self.frame += 1
		if self.frame % 5 == 0:
			if self.AMove == 0:		
				if self.currentAnim == 0:
					self.currentAnim = 1
				else:
					self.currentAnim = 0		
		if self.frame % 3 == 0:		
			if self.AMove == 1:
				if self.currentAnim == 2:
					self.currentAnim = 3
				elif self.currentAnim == 3:
					self.currentAnim = 4
				else:
					self.currentAnim = 2
			if self.AMove == -1:
				if self.currentAnim == 2:
					self.currentAnim = 4
				elif self.currentAnim == 4:
					self.currentAnim = 3
				else:
					self.currentAnim = 2
		if self.frame > self.limit:
			m = Move()
			m.XMove = self.AMove
			m.AMove = self.AMove
			return m
		
		return self
	def draw(self,drawrect,screen):
		return self.currentAnim, drawrect.center
		
class Crouch(object):
	def __init__(self,ps):
		self.AMove = ps.AMove
		self.type = "CROUCH"
	def move(self,rect):
		return (rect.center)		
	def update(self):
		return self
	def draw(self,drawrect,screen):		
		return 8, drawrect.center
